@@ -21,7 +21,7 @@ my $self = new RTF::HTML::Output;
 
 if (@ARGV)  {
   foreach my $filename (@ARGV) {
-    $self->parseStream($filename);
+    $self->parse_stream($filename);
   }
 } else {
   while (<DATA>) {
@@ -30,20 +30,20 @@ if (@ARGV)  {
     print STDERR "-" x 30, "\n";
     print STDERR "RTF string: $_";
     print STDERR "-" x 30, "\n";
-    $self->parseString($_);
+    $self->parse_string($_);
   }
 }
 __END__
 #{} # Ok!
-#{\par} # !Ok
-#{string\par} # !Ok
-#{\b bold {\i italic} bold \par} # !Ok
-#{\b introduction \par } # !Ok
-#{\b first B{\b0 mm{\b b}m}b}
-#{\b first B{\b0 mm{\b b}m}b\par}
-{\i {\b first B{\b0 mm{\b b}m}b\par second B}}
+#{\par} # Ok!
+#{string\par} # Ok!
+#{\b bold {\i italic} bold \par} # Ok!
+#{\b introduction \par } # Ok!
+#{\b first B{\b0 mm{\b b}m}b} #!Ok
+#{\b first B{\b0 mm{\b b}m}b\par} # !Ok
+#{\i {\b first B{\b0 mm{\b b}m}b\par second B}}!Ok
 #{{\par }\b {Introduction\par }}
 #{\pard\plain \b{Introduction\par }}
 #{\b bold \i Bold Italic \i0 Bold again} # Ok!
 #{\b bold {\i Bold Italic }Bold again} # Ok!
-#{\b bold \i Bold Italic \plain\b Bold again} # Ok!
+{\b bold \i Bold Italic \plain\b Bold again} # Ok!
